#include <stdio.h>
#include "orbitersdk.h"
#include "UMMUFB.h"
#include "PAX.h"
#define MFD_LINELENGTH 35

// Constructor
PAX::PAX (DWORD w, DWORD h, VESSEL *vessel)
: MFD (w, h, vessel)
{
	this->vessel=vessel;
	this->object=vessel->GetHandle();
	this->width=(int)w/35;
	this->heigth=(int)h/28;
	lastButton=0;
	lastTime=oapiGetSimTime();
	middleGreen=NULL;
	blink=false;
}

// Destructor
PAX::~PAX ()
{
	// Add MFD cleanup code here
}

// Return button labels
char *PAX::ButtonLabel (int bt)
{
	// The labels for the two buttons used by our MFD mode
	static char *label = "PAX";
	static char *empty = "";
	if (bt == 5) return label;
	return (bt < 5 ? empty : 0);
}

// Return button menus
int PAX::ButtonMenu (const MFDBUTTONMENU **menu) const
{
	// The menu descriptions for the two buttons
	static const MFDBUTTONMENU mnu[6] = {
		{0, 0, 0},
		{0, 0, 0},
		{0, 0, 0},
		{0, 0, 0},
		{0, 0, 0},
		{"Passenger boarding", 0, 'P'}
	};
	if (menu) *menu = mnu;
	return 6; // return the number of buttons used
}

// Repaint the MFD
void PAX::Update (HDC hDC)
{
	char line[2][MFD_LINELENGTH+1];
	OBJHANDLE base;
	DWORD pad;

	SelectDefaultFont (hDC, 0);
	if (middleGreen==NULL)
	{
		LOGPEN pen;
		SelectDefaultPen(hDC, 2);
		GetObject(GetCurrentObject(hDC, OBJ_PEN), sizeof(LOGPEN), &pen);
		middleGreen=pen.lopnColor;
	}
	if (IsUMmuBase(vessel))
	{
		SetTextColor(hDC, RGB(255,255,255));			
		sprintf(line[0], "WELCOME AT TOWER OF", pad);
		char *vesselName=vessel->GetName();
		strncpy(line[1], vesselName, min(MFD_LINELENGTH, strlen(vesselName)));
		CenterTextOut(hDC, 7, line[0]);			
		CenterTextOut(hDC, 9, line[1], -1, true);
		DOCKHANDLE port=vessel->GetDockHandle(0);
		OBJHANDLE pax=vessel->GetDockStatus(port);
		VESSEL *paxv=NULL;
		if (pax!=NULL) paxv=oapiGetVesselInterface(pax);
		if (paxv!=NULL)
		{
			SetTextColor(hDC, RGB(255,255,0));
			CenterTextOut(hDC, 13, vessel->GetName());
		}
		else
		{			
			SetTextColor(hDC, middleGreen);
			CenterTextOut(hDC, 13, "No PAX");
		}
	}
	else
	{
		GetLandedPad(vessel, base, pad);
		if (pad>0)
		{
			SetTextColor(hDC, RGB(255,255,255));			
			sprintf(line[0], "WELCOME AT GATE %d OF", pad);
			oapiGetObjectName(base, line[1], MFD_LINELENGTH);			
			CenterTextOut(hDC, 7, line[0]);			
			CenterTextOut(hDC, 9, line[1], -1, true);			
			if (GetPAX(vessel))
			{
				double time=oapiGetSimTime();
				if ((time-lastTime)>0.5)
				{
					blink=!blink;
					lastTime=time;
				}
				if (blink) SetTextColor(hDC, RGB(255,255,0));
				else SetTextColor(hDC, middleGreen);
				CenterTextOut(hDC, 13, "Ready for PAX");
				SelectDefaultFont (hDC, 1);
				SetTextColor(hDC, middleGreen);
				CenterTextOut(hDC, 26, "Press PAX again after boarding completed", 1);
			}
			else
			{						
				SelectDefaultFont (hDC, 1);
				SetTextColor(hDC, middleGreen);
				CenterTextOut(hDC, 26, "Press PAX for passenger operations", 1);
			}
		}
		else
		{
			DelPAX(vessel);	
			SetTextColor(hDC, middleGreen);
			CenterTextOut(hDC, 13, "NO SERVICE");
		}
	}
}

// MFD message parser
int PAX::MsgProc (UINT msg, UINT mfd, WPARAM wparam, LPARAM lparam)
{
	switch (msg) {
	case OAPI_MSG_MFD_OPENED:
		// Our new MFD mode has been selected, so we create the MFD and
		// return a pointer to it.
		return (int)(new PAX (LOWORD(wparam), HIWORD(wparam), (VESSEL*)lparam));
	}
	return 0;
}

bool PAX::ConsumeKeyBuffered(DWORD key)
{	
	switch(key)
	{
	case OAPI_KEY_P:
		SwitchPAXMode();
		return true;
	default:
		return false;	
	}
}

bool PAX::ConsumeButton(int bt, int event)
{
	if (event==PANEL_MOUSE_LBPRESSED)
	{
		double time=oapiGetSimTime();
		bool re=bt!=lastButton | (time-lastTime)>0.2;
		lastButton=bt;
		lastTime=time;
		if (re)	switch(bt)
		{
		case 5:
			return ConsumeKeyBuffered(OAPI_KEY_P);		
		}		
	}
	return false;
}

void PAX::SwitchPAXMode()
{
	if (GetPAX(vessel)) DelPAX(vessel);
	else SetPAX(vessel);	
	return;
}

void PAX::CenterTextOut(HDC hDC, int line, LPSTR text, int column, bool toUpper)
{
	int l=strlen(text);
	if (toUpper) for(int i=0;i<l;i++) text[i]=toupper(text[i]);
	if (column<0) TextOut(hDC, ((MFD_LINELENGTH-l)/2+1)*width, line*heigth, text, l);
	else TextOut(hDC, column*width, line*heigth, text, l);
}